/**
 * 
 */
package cz.cvut.ok.kapo.paa.sat.awt;

import java.awt.BorderLayout;
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Collection;
import java.util.HashSet;
import java.util.Timer;
import java.util.TimerTask;
import java.util.Vector;

import cz.cvut.ok.kapo.paa.sat.objects.EConstants;
import cz.cvut.ok.kapo.paa.sat.objects.SharedVars;

/**
 * @author karel
 * 
 */
public class GraphDrawer extends Frame {
	private static final long serialVersionUID = 1549146834832548008L;

	GraphCanvas gCanvas = null;

	private Timer timer = null;

	private TimerTask ttask = null;

	/**
	 * 
	 */
	public GraphDrawer() {
		super();
		this.timer = new Timer();
		this.ttask = new RefreshTimerTask();
		this.initialize();
	}

	/**
	 * Vrati instanci GraphCanvas. Pokud jeste nebyla laokovana, tak vytvori
	 * novou.
	 * 
	 * @return GraphCanvas.
	 */
	private GraphCanvas getGraphCanvas() {
		if (this.gCanvas == null) {
			this.gCanvas = new GraphCanvas();
			this.gCanvas.setName("Graph");
		}
		return this.gCanvas;
	}

	/**
	 * Initialize the class.
	 */
	private void initialize() {
		this.setUndecorated(false);
		this.setName("Frame1");
		this.setLayout(new BorderLayout());
		this.setBounds(45, 25, 640, 480);
		this.setTitle("Graph");
		// this.add(getButton1(), getButton1().getName());
		this.add(getGraphCanvas());
		this.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(@SuppressWarnings("unused")
			WindowEvent arg0) {
				doExit();
			}
		});
	}

	/**
	 * Ukonci aplikaci
	 */
	void doExit() {
		System.exit(0);
	}

	/**
	 * Zapne prekreslovani.
	 */
	public void startRefreshing() {
		this.timer.scheduleAtFixedRate(this.ttask, 0L, EConstants.graphRepaintPeriod);
	}

	/**
	 * Vypne prekreslovani.
	 */
	public void stopRefreshing() {
		this.timer.cancel();
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		for (int i = -100; i < 100; i++)
			SharedVars.bestWorkingsStat.add(new Integer((int) (10 * Math
					.exp(i / 2.0))));

		SharedVars.bestSolutionsStat.add(new Integer(12056));
		SharedVars.bestSolutionsStat.add(new Integer(22345325));
		SharedVars.bestSolutionsStat.add(new Integer(1345));
		SharedVars.bestSolutionsStat.add(new Integer(22453056));
		SharedVars.bestSolutionsStat.add(new Integer(1543453));
		SharedVars.bestSolutionsStat.add(new Integer(25345445));
		SharedVars.bestSolutionsStat.add(new Integer(13423467));
		SharedVars.bestSolutionsStat.add(new Integer(24365656));
		SharedVars.bestSolutionsStat.add(new Integer(12055646));

		GraphDrawer gd = new GraphDrawer();
		gd.setVisible(true);
	}

	/**
	 * Canvas, ktery umi na sebe vykreslit graf.
	 * 
	 * @author karel
	 */
	public class GraphCanvas extends Canvas {
		private static final long serialVersionUID = 7208031158049178051L;

		private int yscale;

		public GraphCanvas() {
			super();
			this.yscale = 1;
		}

		/**
		 * Zakresli osy do canvasu g.
		 * 
		 * @param g
		 */
		private void drawAxis(Graphics g) {
			g.drawLine(10, 5, 10, this.getHeight() - 5);
			g.drawLine(5, this.getHeight() - 10, this.getWidth() - 5, this
					.getHeight() - 10);
		}

		private void computeScales() {
			Collection<Integer> set = new HashSet<Integer>();
			synchronized (SharedVars.lock) {
				set.addAll(SharedVars.bestSolutionsStat);
				set.addAll(SharedVars.bestWorkingsStat);
				set.addAll(SharedVars.workingsStat);
			}
			int maxH = this.getHeight() - 15;
			this.yscale = 0;
			// zjistime y-ove meritko (kolika delit, abychom meli graf
			// normalizovany)
			for (Integer tmp : set)
				if (tmp.intValue() > this.yscale)
					this.yscale = tmp.intValue();
			this.yscale = this.yscale / maxH + 1;
			if (this.yscale == 0)
				this.yscale = 1;
		}

		/**
		 * Vykresli graf zadanou barvou. Automaticky zmeni meritko xove osy tak,
		 * aby se prizpusobilo aktualnimu rozsahu. Y-ova osa se meni pouze po 2x
		 * nasobku puvodniho.
		 * 
		 * @param g
		 *            graphics - kam se kresli
		 * @param graph -
		 *            vektor bodu, ktere se vykresli
		 * @param color -
		 *            barva, kterou se kresli
		 */
		private void drawGraph(Graphics g, Vector<Integer> graph, Color color) {
			if (graph.size() <= 0) // prazdny graf nevykreslime
				return;
			Color ctmp = g.getColor();
			g.setColor(color);

			int maxH = this.getHeight() - 15;
			int xoff = 10;
			int yoff = 10;

			int[] xs = new int[graph.size() + 1];
			int[] ys = new int[graph.size() + 1];

			for (int i = 0; i < graph.size(); i++) {
				xs[i] = i + xoff;
				ys[i] = maxH - yoff - (graph.get(i).intValue() / this.yscale);
			}

			g.drawPolyline(xs, ys, graph.size());
			g.setColor(ctmp); // nastavime puvodni arvu pera
		}

		/**
		 * Vykresli graf podle internich promennych.
		 */
		@Override
		public void paint(Graphics g) {
			this.drawAxis(g);
			computeScales(); // prepocitame meritko
			drawGraph(g, SharedVars.cloneBestWorkingsStat(), Color.BLUE);
			drawGraph(g, SharedVars.cloneSolutionsStat(), Color.RED);
			drawGraph(g, SharedVars.cloneWorkingsStat(), Color.GREEN);
		}
	}

	/**
	 * Trida, ktera opakovane bude volat vykresleni tohoto grafu.
	 * 
	 * @author karel
	 * 
	 */
	public class RefreshTimerTask extends TimerTask {

		@Override
		public void run() {
			GraphDrawer.this.gCanvas.repaint();
		}

	}

}
