/**
 * 
 */
package cz.cvut.ok.kapo.paa.sat.engine;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.Reader;
import java.util.SortedSet;
import java.util.StringTokenizer;
import java.util.TreeSet;
import java.util.logging.Level;
import java.util.logging.Logger;

import cz.cvut.ok.kapo.paa.sat.objects.Clausule;
import cz.cvut.ok.kapo.paa.sat.objects.Configuration;
import cz.cvut.ok.kapo.paa.sat.objects.EConstants;
import cz.cvut.ok.kapo.paa.sat.objects.Formula;
import cz.cvut.ok.kapo.paa.sat.objects.SharedVars;
import cz.cvut.ok.kapo.paa.sat.objects.State;

/**
 * @author karel
 *
 */
public class SATApp {
	public Logger Log;
	private Reader fin;
	private Configuration conf;
	private SimulatedAnnealingEngine engine;
	
	/**
	 * Nastavi soubor, odkud se bude nacitat konfigurace, logger
	 * a zavola inicializacni metodu.
	 * @param confFile konfiguracni soubor.
	 */
	public SATApp(Reader confFile){
		this.Log = Logger.getLogger(EConstants.APP_LOG);
		this.fin = confFile;
		this.init();
	}
	
	/**
	 * Nastavi pocatecni hodnoty, nacte konfiguraci a inicializuje engine.
	 */
	protected void init(){
		this.conf = new Configuration();
		this.conf.setAlpha(EConstants.alpha);
		this.conf.setMaxTemp(100.0);
		this.conf.setMinTemp(1.0);
		this.readFormula(this.conf);
		this.engine = new SimulatedAnnealingEngine(true);
		this.engine.initialize(this.conf, EConstants.completeTimeout); //timeout 20s
	}
	
	/**
	 * Nacte formuli z konfiguracniho souboru. Zavre soubor.
	 * @return nactenou formuli. V pripade chyby null.
	 */
	protected Formula readFormula(Configuration conf){
		Formula formula=new Formula();
		BufferedReader br = new BufferedReader(this.fin);
		Clausule cls = new Clausule();
		
		String line;
		StringTokenizer st;
		int numCls=0, //pocet klausuli
		    numVars=0,//pocel promennych
		    n=0,	  //realny pocet klausuli
		    weights[]=null;//vahy promennych
		boolean firstLine=true;
		boolean secondLine=true;
		try {
			while ( (line=br.readLine())!= null){
				String trimmed = line.trim().toLowerCase();
				if (trimmed.startsWith("#") 
						|| trimmed.startsWith("c") 
						|| trimmed.equals(""))
					continue; //komentar v souboru
				st = new StringTokenizer(trimmed, " ");
				
				if (firstLine){ //pokud je tohle prvni radek
					if ( !st.nextToken().equals("p")
							|| !st.nextToken().equals("cnf"))
						throw new IOException("Prvni radek nezacina: 'p cnf' !!");
					numVars = Integer.valueOf(st.nextToken()).intValue();
					numCls =  Integer.valueOf(st.nextToken()).intValue();
					firstLine=false;
					weights = new int[numVars];
					continue; //dalsi radek
				}
				if (secondLine){ //pokud je tohle prvni radek
					if ( !st.nextToken().equals("weights") )
						throw new IOException("Druhy radek nezacina: 'weights' !!");
					for (int j=0; j<numVars; j++)
						weights[j]=Integer.valueOf(st.nextToken()).intValue();
					secondLine=false;
					continue; //dalsi radek
				}
				
				//parsovani klausuli
				
				while (st.hasMoreTokens()){
					int tmp = Integer.valueOf(st.nextToken()).intValue();
					if (Math.abs(tmp) > numVars)
						throw new IOException(n + ". klausule obsahuje odkaz na promennou mimo rozsah ("+tmp+") !!");

					if (tmp==0){	//klausule ukonceny znakem 0
						n++;
						this.Log.log(Level.INFO, "Naparsovana klausule: "+cls.toString());
						formula.addClausule(cls);
						cls = new Clausule();
					}
					else
						cls.addVariable(Math.abs(tmp), weights[Math.abs(tmp)-1], tmp<0);
				}
			}
		} catch (IOException e) {
			this.Log.log(Level.SEVERE, "Chyba pri parsovani konfigurace:\n+->" + e.getMessage());
			return null;
		}
		
		if (n!=numCls)
			this.Log.log(Level.INFO, "Chyba pri parsovani konfigurace:Nesedi pocet klasuli!");
		
		conf.setFormula(formula);
		conf.setNumVars(numVars);
		
		try {
			br.close();
		} catch (IOException e) {
			this.Log.log(Level.SEVERE, "Chyba pri uzavirani konfiguracniho souboru!\n +->"+e.getMessage());
		}
		return formula;
	}
	
	public void execute(){
		this.engine.run();
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		SATApp app;
		if (args.length < 1){
			System.out.println("Chybi argument! (jmeno souboru se zadanim)");
			return;
		}
		
		try {
			FileReader fr = new FileReader(args[0]);
			app = new SATApp(fr);
		} catch (FileNotFoundException e) {
			e.printStackTrace();
			return;
		}
		
		app.execute();

		app.Log.log(Level.INFO, "Pocet reseni: "+ SharedVars.solutions.size());

		for (State state: SharedVars.solutions){
			app.Log.log(Level.INFO, "Solution: "+ state.toString() + "=" + app.conf.getFormula().getCost(state)+" ("+app.conf.getFormula().getFormulaWeight(state)+")");
		}
		SortedSet ss = new TreeSet<Integer>(SharedVars.bestSolutionsStat);
		app.Log.log(Level.INFO, "solution stat costs: "+ ss.toString());
		
	/*	for (State state: SharedVars.bestWorkings){
			app.Log.log(Level.INFO, "Working: "+ state.toString());
		}
		for (Integer state: SharedVars.bestSolutionsStat){
			app.Log.log(Level.INFO, "Solution stat: "+ state.toString());
		}
		for (Integer state: SharedVars.bestWorkingsStat){
			app.Log.log(Level.INFO, "Working stat: "+ state.toString());
		}*/

	}
}