/**
 * 
 */
package cz.cvut.ok.kapo.paa.sat.engine;

import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.Level;
import java.util.logging.Logger;

import cz.cvut.ok.kapo.paa.sat.objects.Configuration;
import cz.cvut.ok.kapo.paa.sat.objects.EConstants;


/**
 * @author karel
 *
 */
public class SAThread extends Thread {
	/** aplikacni log */
	private Logger Log = Logger.getLogger(EConstants.APP_LOG);
	private SAEngine engine;
	private boolean stop;
	
	protected double maxTemp, minTemp, temperature, alpha;
 
	
	/**
	 * 
	 * @param conf
	 */
	public SAThread(){
		this.stop=false;
		this.minTemp=1.0; 
	}
	
	/**
	 * Inicializuje vlakno.
	 * @param conf
	 */
	public void initialize(Configuration conf){
		this.engine = new SAEngine(conf);
		this.maxTemp = conf.getMaxTemp();
		this.minTemp = conf.getMinTemp();
		this.alpha = conf.getAlpha();
		this.temperature = this.engine.findStartingTemperature(conf.getMaxTemp(), EConstants.initialFraction);
	}
	
	/**
	 * Zastavi vypocet vlakna po prave probihajici iteraci.
	 */
	public void stopCompute(){
		this.Log.log(Level.INFO, "Zachycen signal stop!");
		this.stop=true;
	}
	
	/**
	 * Vrati ochlazenou teplotu.
	 * @param temperature z jake teploty se ochazuje.
	 */
	public double cool(double temperature){
		return temperature * this.alpha;
	}

	@Override
	public void run(){
		Timer timer = new Timer();
		FillTimerTask ftt = new FillTimerTask(this.engine);
		timer.scheduleAtFixedRate(ftt, 0L, EConstants.graphFillPeriod);
		
		int iters = EConstants.maxIters;
		while(!this.stop && !this.engine.frozen()){
			this.Log.log(Level.INFO, "thread: "+getName()+"; teplota: "+this.temperature +" iteraci: "+iters);
			this.engine.runPart(this.temperature, iters);
			this.temperature = this.cool(this.temperature);
			iters *= EConstants.aIters; //se snizujici teplotou zvysujeme pocet iteraci
			yield();
		}
		timer.cancel();
		this.Log.log(Level.INFO,"thread: "+getName()+"; koncim vlakno (stop:"+this.stop+")");
	}
	
	/**
	 * Periodicky vola ukladani statistik
	 * @author karel
	 */
	public class FillTimerTask extends TimerTask{
		private SAEngine core;
		public FillTimerTask(SAEngine engine){
			super();
			this.core = engine;
		}
		@Override
		public void run() {
			this.core.fillStats();
		}
		
	}
}
