/**
 * Balicek obsahuje tridy pro metodu simulovane ochlazovani.
 */
package cz.cvut.ok.kapo.paa.sat.engine;

import java.util.Timer;
import java.util.logging.Level;
import java.util.logging.Logger;

import cz.cvut.ok.kapo.paa.sat.awt.GraphDrawer;
import cz.cvut.ok.kapo.paa.sat.objects.Configuration;
import cz.cvut.ok.kapo.paa.sat.objects.EConstants;

/**
 * Simulovane ochlazovani. Protoze se jedna o iterativni metodu, musime nejak zabezpecit,
 * abychom ji mohli stopnout po nejake dobe. Toho dosahneme tim, ze implementujeme vlastni
 * algoritmus jako vlakno. @see SAThread
 * @author karel
 *
 */
public class SimulatedAnnealingEngine {
	/** Logger */
	Logger Log;
	/** Vypocetni vlakno */
	private SAThread core;
	/** Casovy limit pro celkovy vypocet v ms */
	protected long timeLimit;

	protected GraphDrawer agd = null;
	
	/**
	 * @param conf
	 */
	public SimulatedAnnealingEngine(boolean showStats) {
		super();
		this.Log  = Logger.getLogger(EConstants.APP_LOG);
		if (showStats){
			this.agd = new GraphDrawer();
		}
	}
	
	/**
	 * Provede inicializaci enginu. Vytvori nove vlakno s patricnou konfiguraci.
	 * @param conf konfigurace - SAT formule
	 * @param timelimit - cas pro celkovy vypocet [ms]
	 */
	public void initialize(Configuration conf, long timelimit){
		this.timeLimit = timelimit;
		this.core = new SAThread();
		this.core.initialize(conf);
	}

	/**
	 * Spusti vypocet problemu.
	 */
	public void run() {
		System.out.println("SAEngine nastartovan");
		this.agd = new GraphDrawer();
		this.agd.setVisible(true);
		this.agd.startRefreshing();
		//long start = System.currentTimeMillis();
		Timer timer = new Timer();
		SATerminateTask tt = new SATerminateTask(this.core);
		timer.schedule(tt, this.timeLimit);
		this.core.start();	//pustime vlakno
		
		try { // a pockame az skonci
			this.Log.log(Level.INFO, "Cekam na vypocetni vlakno...");
			this.core.join();
			tt.cancel();
			timer.cancel();
			this.Log.log(Level.INFO, "Vypocetni vlakno skoncilo.");
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		this.agd.stopRefreshing();
		
	}

}
