package cz.cvut.ok.kapo.paa.sat.objects;

import java.util.Vector;

/**
 * Reprezentuje 1 klauzuli formule. Obsahuje i vahy jednotlivych promennych.
 * Klausule muze mit libovolny pocet promennych.
 * Obsahuje metody na vyhodnoceni jak ceny klausule, tak logicke hodnoty klausule.
 * @author karel
 *
 */
public class Clausule {
	/**
	 * Jednotlive promenne klausule.
	 */
	private Vector<Variable> variables;
	
	public Clausule() {
		super();
		this.variables = new Vector<Variable>();
	}
	
//========================= manipulacni metody ==================================
	
	/**
	 * Prida promennou ke klausuli.
	 * @param varNum cislo promenne
	 * @param value hodnota promenne: <br>abs(value) = vaha promenne
	 *                                <br>sig(value) = negace promenne.
	 */
	public void addVariable(int varNum, int value){
		addVariable( varNum, Math.abs(value), (value<0) );
	}
	
	/**
	 * Prida promennou ke klausuli.
	 * @param num - cislo promenne ( >0 )
	 * @param weight - vaha promenne ( >0 )
	 * @param neg - je promenna negovana? (1 ne, -1 ano)
	 */
	public void addVariable(int num, int weight, boolean neg){
		Variable tmp = new Variable();
		tmp.setVarNum(num);
		tmp.setVarNeg(neg);
		tmp.setVarWeight(weight);
		this.variables.add(tmp);
	}
	
	/**
	 * Odstrani danou promennou z klauzule. Pokud neexistuje, nic se nedeje.
	 * @param varNum cislo promenne, ktera se ma odstranit.
	 */
	public void removeVariable(int varNum){
		for (Variable item: this.variables){
			if (item.getVarNum() == varNum){
				this.variables.remove(item);
				break;
			}
		}
	}
	
	/**
	 * Vrati promennou na dane pozici.
	 * @param i
	 * @return
	 */
	public Variable getVariable(int idx){
		return this.variables.get(idx);
	}
	
//========================= ohodnocovaci metody =============================
	/**
	 * Vrati booleovskou hodnotu klauzule. Ohodnoceni jednotlivych promennych
	 * se bere z parametru State.
	 * @param state - ohodnoceni jednotlivych promennych.
	 * @return true/false
	 */
	public boolean getValue(State state){
		boolean value = false;
		for (Variable var: this.variables){
			value = value || var.getRealValue(state);
		}
		return value;
	}
	
	/**
	 * Vrati vahu klausule podle ohodnocenych promennych. Ohodnoceni promennych
	 * se bere z parametru State.
	 * @param state - ohodnoceni jednotlivych promennych
	 * @return - celkova vaha ohodnocene formule
	 */
	public int getWeight(State state){
		int value=0;
		for (Variable var: this.variables){
			if (state.getValue(var.getVarNum()))
				value+=var.getVarWeight();
		}
		return value;
	}
	
	/**
	 * Vrati maximalni vahu klausule. Tj. predpoklada,
	 * ze vsechny promenne jsou true.
	 * @param state - ohodnoceni jednotlivych promennych
	 * @return - maximalni vaha ohodnocene formule
	 */
	public int getMaxWeight(){
		int value=0;
		for (Variable var: this.variables){
			value+=var.getVarWeight();
		}
		return value;
	}
	
// ======================= statisticke metody ===============================
	/**
	 * @return Vrati pocet promennych v klauzuli.
	 */
	public int getVariablesCount(){
		return this.variables.size();
	}
	
	/**
	 * Vrati klausuli naformatovanou do formatu:
	 * ( 3x4 1x3' 3x8 )
	 * 
	 * Prvni cislo je vaha, druhe cislo je poradove cislo promenne.
	 * Apostrof je negace.
	 */
	@Override
	public String toString(){
		StringBuffer sb = new StringBuffer();
		
		sb.append("( ");
		for (Variable var: this.variables){
			sb.append(var.toString());
		}
		sb.append(" )");
		
		return sb.toString();
	}	
}
