 package cz.cvut.ok.kapo.paa.sat.objects;

import java.util.Vector;

/**
 * Obsahuje reprezentaci SAT formule.
 * 
 * @author karel
 *
 */
/**
 * @author karel
 *
 */
public class Formula {
	/** obsahuje seznam klauzuli */
	private Vector<Clausule> clausules;
	/** maxValue */
	private int maxValue;

	
	public Formula() {
		super();
		this.clausules=new Vector<Clausule>();
		this.maxValue=0;
	}

	
	/**
	 * Prida klauzuli k teto formuli.
	 * @param cls
	 */
	public void addClausule(Clausule cls){
		this.clausules.add(cls);
		this.maxValue += cls.getMaxWeight();
	}
	
	/**
	 * Vrati ohodnoceni cele formule.
	 * @param state - ohodnoceni promenych
	 */
	public int getFormulaWeight(State state){
		int value=0;
		for (Clausule cls: this.clausules){
			value += cls.getWeight(state);
		}
		return value;
	}
	
	/**
	 * Vrati true/false : hodnotu formule.
	 * @param state ohodnoceni promennych.
	 */
	public boolean getFormulaValue(State state){
		boolean value=true;
		for (Clausule cls: this.clausules){
			value = value && cls.getValue(state);
		}
		return value;
	}
	
	/**
	 * Vrati pomer clausuli/pocet promennych.
	 * @param state
	 */
	public double getFormulaFraction(State state){
		return (double)this.clausules.size() / (double)state.getVarsCount();
	}
	
	/**
	 * Vrati maximalni moznou hodnotu formule. 
	 */
	public int getMaxFormulaWeight(){
		return this.maxValue;
	}
	
	/**
	 * @return pocet vsech klausuli.
	 */
	public int getClausulesCount(){
		return this.clausules.size();
	}
	
	/**
	 * Pocet pravdivych klausuli
	 * @param state ohodnoceni promennych
	 */
	public int getRightClausulesCount(State state){
		int count=0;
		for (Clausule cls: this.clausules){
			if (cls.getValue(state))
				count++;
		}
		return count;
	}
	
	/**
	 * Vrati ohodnoceni daneho stavu. Optimalizacni kriterium!
	 * Cim vetsi, tim je stav ohodnocen lepe.
	 * 
	 * Bere v uvahu i to, zda je stav resenim, nebo ne.
	 */
	public double getCost(State state){
		if (state==null)
			return 0.5;
		double cls = (double)this.getRightClausulesCount(state)/this.getClausulesCount();
		double weight = (double)this.getFormulaWeight(state)/this.getMaxFormulaWeight();
		return (100.0*(0.3*weight + Math.exp(cls))); // cls cast je <1; 2.7>, weight <0; 0.6>
		//return cls+weight;
	}

}
